if exists(select 1 from sysobjects where [id] = object_id('sp_check_columns') and xtype = 'P')
begin
  drop procedure sp_check_columns
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_objects') and xtype = 'P')
begin
  drop procedure sp_check_objects
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_procedure') and xtype = 'P')
begin
  drop procedure sp_check_procedure
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_foreignkey') and xtype = 'P')
begin
  drop procedure sp_check_foreignkey
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_object_script') and xtype = 'P')
begin
  drop procedure sp_check_object_script
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_indexes') and xtype = 'P')
begin
  drop procedure sp_check_indexes
end
go

if exists(select 1 from sysobjects where [id] = object_id('sp_check_object_permissions') and xtype = 'P')
begin
  drop procedure sp_check_object_permissions
end
go

create procedure sp_check_object_permissions @object_name varchar(50) as
begin
  declare @o_id int, @o_action int, @o_protecttype int, @o_columns varbinary(4000)
  declare @c_id int, @c_action int, @c_protecttype int, @c_columns varbinary(4000)

  set @o_id = object_id('etalondb..' + @object_name)
  set @c_id = object_id(@object_name)

  if (@o_id is null) or (@c_id is null)
  begin
    select 'ERROR! in calling of sp_check_object_permissions for object: ' + @object_name
    return
  end

  declare o_sysprotects cursor fast_forward for
    select action, protecttype, [columns] from etalondb..sysprotects
    where [id] = @o_id

  open o_sysprotects

  fetch o_sysprotects into @o_action, @o_protecttype, @o_columns

  while @@fetch_status = 0
  begin
    select @c_action = action, @c_protecttype = protecttype, @c_columns = [columns] 
    from sysprotects where [id] = @c_id and action = @o_action

    if (@c_action is null) or (@o_action <> @c_action) or (@o_protecttype <> @c_protecttype)
      or (@o_columns <> @c_columns)
    begin
      select 'ERROR! in permissions: etalondb.' + @object_name, @o_action, @o_protecttype, @o_columns,
        @c_action, @c_protecttype, @c_columns
    end

    fetch o_sysprotects into @o_action, @o_protecttype, @o_columns
  end

  close o_sysprotects
  deallocate o_sysprotects
end
go

create procedure sp_check_object_script @object_name varchar(50) as
begin
  return

  declare @o_len int, @c_len int

  select @o_len = len([ctext]) from etalondb..syscomments where [id] = object_id('etalondb..' + @object_name)
  select @c_len = len([ctext]) from syscomments where [id] = object_id(@object_name)

  if (@o_len is null) or (@c_len is null)
  begin
    select 'ERROR! in calling of sp_check_object_script for object: ' + @object_name
    return
  end

  if (@o_len <> @c_len)
  begin
    select 'ERROR! definitions of ' + @object_name + ' are differ', @o_len as [etalondb length], @c_len as [check length]
  end
end
go

create procedure sp_check_indexes @full_table_name varchar(50) as
begin
  create table #indexdb (indexname varchar(50) null, description varchar(2048) null ,indexcols varchar(2048) null)

  declare @o_table_id int, @o_indid int, @o_groupid int, @o_name varchar(50), @o_status int, @o_keys varchar(2048),
    @o_description varchar(2048)
  declare @c_table_id int, @c_indid int, @c_groupid int, @c_name varchar(50), @c_status int, @c_keys varchar(2048),
    @c_description varchar(2048)

  declare @i int, @thiskey varchar(50)

  declare @empty varchar(1) select @empty = ''
  declare @des1	varchar(35), @des2 varchar(35),	@des4 varchar(35), @des32 varchar(35), @des64 varchar(35),
    @des2048 varchar(35), @des4096 varchar(35),	@des8388608 varchar(35), @des16777216 varchar(35)

  select @des1 = [name] from master..spt_values where type = 'I' and number = 1
  select @des2 = [name] from master..spt_values where type = 'I' and number = 2
  select @des4 = [name] from master..spt_values where type = 'I' and number = 4
  select @des32 = [name] from master..spt_values where type = 'I' and number = 32
  select @des64 = [name] from master..spt_values where type = 'I' and number = 64
  select @des2048 = [name] from master..spt_values where type = 'I' and number = 2048
  select @des4096 = [name] from master..spt_values where type = 'I' and number = 4096
  select @des8388608 = [name] from master..spt_values where type = 'I' and number = 8388608
  select @des16777216 = [name] from master..spt_values where type = 'I' and number = 16777216

  set @o_table_id = object_id('etalondb..' + @full_table_name)
  set @c_table_id = object_id(@full_table_name)

  declare o_sysindexes cursor fast_forward for
    select indid, groupid, [name], status
    from etalondb..sysindexes 
    where [id] = @o_table_id and indid > 0 and indid < 255 and (status & 64) = 0

  declare c_sysindexes cursor fast_forward for
    select indid, groupid, [name], status
    from sysindexes
    where [id] = @c_table_id and indid > 0 and indid < 255 and (status & 64) = 0

  open c_sysindexes
  fetch c_sysindexes into @c_indid, @c_groupid, @c_name, @c_status

  while @@fetch_status = 0
  begin
    select @c_keys = index_col(@full_table_name, @c_indid, 1), @i = 2,
      @thiskey = index_col(@full_table_name, @c_indid, 2)
    while (@thiskey is not null)
    begin
      select @c_keys = @c_keys + ',' + @thiskey, @i = @i + 1
      select @thiskey = index_col(@full_table_name, @c_indid, @i)
    end

    select @thiskey = groupname from sysfilegroups where groupid = @c_groupid
    set @c_description = convert(varchar(210),
      case when (@c_status & 16)<>0 then 'clustered' else 'nonclustered' end
      + case when (@c_status & 1)<>0 then ', '+@des1 else @empty end
      + case when (@c_status & 2)<>0 then ', '+@des2 else @empty end
      + case when (@c_status & 4)<>0 then ', '+@des4 else @empty end
      + case when (@c_status & 64)<>0 then ', '+@des64 else case when (@c_status & 32)<>0 then ', '+@des32 else @empty end end
      + case when (@c_status & 2048)<>0 then ', '+@des2048 else @empty end
      + case when (@c_status & 4096)<>0 then ', '+@des4096 else @empty end
      + case when (@c_status & 8388608)<>0 then ', '+@des8388608 else @empty end
      + case when (@c_status & 16777216)<>0 then ', '+@des16777216 else @empty end
      + ' located on ' + @thiskey)

    insert #indexdb values (@c_name, @c_description, @c_keys)

    fetch c_sysindexes into @c_indid, @c_groupid, @c_name, @c_status
  end

  close c_sysindexes
  deallocate c_sysindexes

  open o_sysindexes
  fetch o_sysindexes into @o_indid, @o_groupid, @o_name, @o_status

  while @@fetch_status = 0
  begin
    select @o_keys = index_col('etalondb..' + @full_table_name, @o_indid, 1), @i = 2,
      @thiskey = index_col('etalondb..' + @full_table_name, @o_indid, 2)
    while (@thiskey is not null)
    begin
      select @o_keys = @o_keys + ',' + @thiskey, @i = @i + 1
      select @thiskey = index_col('etalondb..' + @full_table_name, @o_indid, @i)
    end

    select @thiskey = groupname from etalondb..sysfilegroups where groupid = @o_groupid
    set @o_description = convert(varchar(210),
      case when (@o_status & 16)<>0 then 'clustered' else 'nonclustered' end
      + case when (@o_status & 1)<>0 then ', '+@des1 else @empty end
      + case when (@o_status & 2)<>0 then ', '+@des2 else @empty end
      + case when (@o_status & 4)<>0 then ', '+@des4 else @empty end
      + case when (@o_status & 64)<>0 then ', '+@des64 else case when (@o_status & 32)<>0 then ', '+@des32 else @empty end end
      + case when (@o_status & 2048)<>0 then ', '+@des2048 else @empty end
      + case when (@o_status & 4096)<>0 then ', '+@des4096 else @empty end
      + case when (@o_status & 8388608)<>0 then ', '+@des8388608 else @empty end
      + case when (@o_status & 16777216)<>0 then ', '+@des16777216 else @empty end
      + ' located on ' + @thiskey)

    set @c_name = null
    select @c_name = indexname, @c_description = description, @c_keys = indexcols from #indexdb 
    where indexcols = @o_keys and description = @o_description

    if (@c_name is not null)
    begin
      if (@o_name <> @c_name)
      begin
        select 'WARNING! index names are differ: etalondb.' + @o_name, @c_name
      end
    end else
    begin
      select 'ERROR! index ' + @o_name + ' not found'
    end

    fetch o_sysindexes into @o_indid, @o_groupid, @o_name, @o_status
  end

  close o_sysindexes
  deallocate o_sysindexes

  drop table #indexdb
end
go

create procedure sp_check_foreignkey @const_name varchar(50) as
begin
  declare @o_foreign_table varchar(50), @o_rel_table varchar(50), @o_foreign_key varchar(50), @o_rel_key varchar(50)
  declare @c_foreign_table varchar(50), @c_rel_table varchar(50), @c_foreign_key varchar(50), @c_rel_key varchar(50)

  select @o_foreign_table = fo.[name], @o_rel_table = ro.[name],
    @o_foreign_key = fc.[name], @o_rel_key = rc.[name]
  from etalondb..sysforeignkeys f, etalondb..sysobjects fo, etalondb..sysobjects ro,
    etalondb..syscolumns fc, etalondb..syscolumns rc
  where fo.[id] = f.fkeyid and ro.[id] = f.rkeyid 
    and fc.[id] = fo.[id] and fc.colid = f.fkey
    and rc.[id] = ro.[id] and rc.colid = f.rkey
    and f.constid = object_id('etalondb..' + @const_name)

  select @c_foreign_table = fo.[name], @c_rel_table = ro.[name],
    @c_foreign_key = fc.[name], @c_rel_key = rc.[name]
  from sysforeignkeys f, sysobjects fo, sysobjects ro,
    syscolumns fc, syscolumns rc
  where fo.[id] = f.fkeyid and ro.[id] = f.rkeyid 
    and fc.[id] = fo.[id] and fc.colid = f.fkey
    and rc.[id] = ro.[id] and rc.colid = f.rkey
    and f.constid = object_id(@const_name)

  if (@c_foreign_table is null) or (@o_foreign_table <> @c_foreign_table) or (@o_rel_table <> @c_rel_table)
    or (@o_foreign_key <> @c_foreign_key) or (@o_rel_key <> @c_rel_key)
  begin
    select 'ERROR! etalondb.' + @const_name as [constraint name],
      @o_foreign_table as [foreign table], @o_rel_table as [rel table],
      @o_foreign_key as [foreign key], @o_rel_key as [rel key]

    select 'checked table     ' + @const_name as [constraint name],
      @c_foreign_table as [foreign table], @c_rel_table as [rel table],
      @c_foreign_key as [foreign key], @c_rel_key as [rel key]
  end
end
go

create procedure sp_check_procedure @proc_name varchar(50) as
begin
  declare @o_proc_id int, @o_param_name varchar(50), @o_param_type_name varchar(50), @o_param_length int,
    @o_isoutparam int
  declare @c_proc_id int, @c_param_name varchar(50), @c_param_type_name varchar(50), @c_param_length int,
    @c_isoutparam int

  set @o_proc_id = object_id('etalondb..' + @proc_name)
  set @c_proc_id = object_id(@proc_name)

  if (@o_proc_id is null) or (@c_proc_id is null)
  begin
    select 'ERROR! in calling of sp_check_procedure for object: ' + @proc_name
    return
  end

  declare o_syscolumns cursor fast_forward for
    select c.[name], t.[name], c.length, c.isoutparam
    from etalondb..syscolumns c, etalondb..systypes t
    where c.usertype = t.usertype and c.[id] = @o_proc_id

  open o_syscolumns

  fetch o_syscolumns into @o_param_name, @o_param_type_name, @o_param_length, @o_isoutparam

  while @@fetch_status = 0
  begin
    select @c_param_name = null

    select @c_param_name = c.[name], @c_param_type_name = t.[name], @c_param_length = c.length,
      @c_isoutparam = c.isoutparam
    from syscolumns c, systypes t
    where c.usertype = t.usertype and c.[id] = @c_proc_id and c.[name] = @o_param_name

    if (@c_param_name is null) or (@o_param_type_name <> @c_param_type_name)
      or (@o_param_length <> @c_param_length) or (@o_isoutparam <> @c_isoutparam)
    begin
      select 'ERROR! etalondb.' + @proc_name + '.' + 
        @o_param_name + '-' + @o_param_type_name as [params info], @o_param_length as length, @o_isoutparam as isout,
        @c_param_name + '-' + @c_param_type_name as [params info], @c_param_length as length, @c_isoutparam as isout
    end

    fetch o_syscolumns into @o_param_name, @o_param_type_name, @o_param_length, @o_isoutparam
  end

  close o_syscolumns
  deallocate o_syscolumns
end
go

create procedure sp_check_columns @table_name varchar(50) as
begin
  declare @o_name varchar(50), @o_table_id int, @o_type_name varchar(50), @o_length int, @o_isnullable int, @o_status int
  declare @c_name varchar(50), @c_table_id int, @c_type_name varchar(50), @c_length int, @c_isnullable int, @c_status int

  set @o_table_id = object_id('etalondb..' + @table_name)
  set @c_table_id = object_id(@table_name)

  if (@o_table_id is null) or (@c_table_id is null)
  begin
    select 'ERROR! in calling of sp_check_columns for object: ' + @table_name
    return
  end

  declare o_syscolumns cursor fast_forward for
    select c.[name], t.[name], c.length , c.isnullable, c.status
    from etalondb..syscolumns c, etalondb..systypes t
    where c.usertype = t.usertype and c.[id] = @o_table_id

  open o_syscolumns

  fetch o_syscolumns into @o_name, @o_type_name, @o_length, @o_isnullable, @o_status

  while @@fetch_status = 0
  begin
    select @c_name = null

    select @c_name = c.[name], @c_type_name = t.[name],
      @c_length = c.length, @c_isnullable = c.isnullable, @c_status = c.status
    from syscolumns c, systypes t
    where (c.usertype = t.usertype) and (c.[id] = @c_table_id) and (c.[name] = @o_name)

    if (@c_name is null) or (@o_type_name <> @c_type_name) or (@o_length <> @c_length)
      or (@o_isnullable <> @c_isnullable) or (@o_status <> @c_status)
    begin
      select 'ERROR! etalondb.' + @table_name + '.' + 
        @o_name + '-' + @o_type_name as [column info], @o_length as length, @o_isnullable as isnullable, @o_status as status,
        @c_name + '-' + @c_type_name as [column info], @c_length as length, @c_isnullable as isnullable, @c_status as status
    end

    fetch o_syscolumns into @o_name, @o_type_name, @o_length, @o_isnullable, @o_status
  end

  close o_syscolumns
  deallocate o_syscolumns
end
go

create procedure sp_check_objects as
begin
  declare @o_name varchar(50), @o_id int, @o_type char(2)
  declare @c_name varchar(50), @c_id int

  declare o_sysobjects cursor fast_forward for
    select [name], [id], [type] from etalondb..sysobjects 
    where
      (type in ('C', 'F', 'P', 'R', 'TR', 'U', 'V')) order by [name]

  open o_sysobjects

  fetch o_sysobjects into @o_name, @o_id, @o_type

  while @@fetch_status = 0
  begin
    select @c_name = null, @c_id = null

    select @c_name = [name], @c_id = [id] from sysobjects 
    where [name] = @o_name and isnull(objectproperty([id], 'ISMSShipped'), 1) <> 1

    if (@c_name is not null)
    begin
      if (@o_type in ('C', 'P', 'TR', 'V'))
      begin
        exec sp_check_object_script @c_name
      end

      if (@o_type in ('U', 'P', 'TR', 'V'))
      begin
        exec sp_check_object_permissions @c_name
      end

      if (@o_type = 'U')
      begin
        exec sp_check_columns @c_name
        exec sp_check_indexes @c_name
      end else
      if (@o_type = 'V')
      begin
        exec sp_check_columns @c_name
      end else
      if (@o_type = 'P')
      begin
        exec sp_check_procedure @c_name
      end else
      if (@o_type = 'F')
      begin
        exec sp_check_foreignkey @c_name
      end

      select 'OK! ' + @o_type + '-' + @c_name
    end else
    if (isnull(objectproperty(@c_id, 'ISMSShipped'), 1) <> 1)
    begin
      select 'ERROR! etalondb.' + @o_type + '-' + @o_name
    end

    fetch o_sysobjects into @o_name, @o_id, @o_type
  end

  close o_sysobjects
  deallocate o_sysobjects
end
go
